unit CurrentTrackInfoToFile_Impl;

interface

uses
  Windows,
  Messages,
  JclUnicode,
  AIMPAddonCustomPlugin, AIMPSDKCore, AIMPSDKAddons;

type
  { TAIMPHook }

  TAIMPHook = class(TInterfacedObject, IAIMPCoreUnitMessageHook)
  private
    FPlugin: TAIMPAddonsCustomPlugin;
    procedure CoreMessage(AMessage: DWORD; AParam1: Integer; AParam2: Pointer; var AResult: HRESULT); stdcall;
  public
    destructor Destroy; override;
    property Plugin: TAIMPAddonsCustomPlugin read FPlugin write FPlugin default nil;
  end;

  { TAIMPAddonDemoPlugin }

  TAIMPAddonDemoPlugin = class(TAIMPAddonsCustomPlugin)
  private
    FMenuHandle: HAIMPMENU;
    FHook: TAIMPHook;
    FPlaylistManager: IAIMPAddonsPlaylistManager;
    FFilesList: TWideStringList;
    FPlayerState: Integer;
    //procedure MenuFinalize;
    //procedure MenuInitialize;
    procedure PlayerFinalize;
    procedure PlayerInitialize;
    //
    procedure OnSettingsChanged;
  protected
    function GetPluginAuthor: PWideChar; override; stdcall;
    function GetPluginInfo: PWideChar; override; stdcall;
    function GetPluginName: PWideChar; override; stdcall;
    function GetPluginFlags: Cardinal; override; stdcall;
    function Initialize(ACoreUnit: IAIMPCoreUnit): HRESULT; override; stdcall;
    function Finalize: HRESULT; override; stdcall;
    function ShowSettingsDialog(AParentWindow: HWND): HRESULT; override; stdcall;
  public
    property PlaylistManager: IAIMPAddonsPlaylistManager read FPlaylistManager default nil;
    property PlayerState: Integer read FPlayerState write FPlayerState default 0;
    procedure UpdateCurrentTrack(const Text: PWideChar);
  end;

implementation

{.$R RandomPlaylistIcons.RES}

uses
  Classes,
  SysUtils,
  CurrentTrackInfoToFile_Defines,
  CurrentTrackInfoToFile_SetupForm{,
  gnugettext,
  languagecodes,
  Locales};

procedure _MenuClick(AUserData: TAIMPAddonDemoPlugin; AHandle: HAIMPMENU); stdcall;
begin
   AUserData.ShowSettingsDialog(0);
end;

{ TAIMPAddonDemoPlugin }

function TAIMPAddonDemoPlugin.Finalize: HRESULT;
begin
  //
  if Assigned(SF) then
   begin
    if SF.Showing then
      SF.Close;
    FreeAndNil(SF);
   end;
  //
  CoreUnit.MessageUnhook(FHook);
  FHook.Plugin:=nil;
  FHook:=nil;
  //
  if Assigned(FFilesList) then
    FreeAndNil(FFilesList);
  //
  if Assigned(FPlaylistManager) then
    FPlaylistManager:=nil;
  //
  //MenuFinalize;
  //
  PlayerFinalize;
  //
  Result := inherited Finalize;
end;

function TAIMPAddonDemoPlugin.Initialize(ACoreUnit: IAIMPCoreUnit): HRESULT;
begin
  Result := inherited Initialize(ACoreUnit);
  if Succeeded(Result) then
  begin
    PlayerInitialize;
    {
    textdomain(myPluginName);
    myLocales:=TLocales.Create(PluginsFolder);
    }
    //
    FMenuHandle:=nil;
    FFilesList:=nil;
    //
    OnSettingsChange:=OnSettingsChanged;
    LoadSettings(Self, myPluginName, mySettings);
    SettingsChanged(mySettings);
    //
    //MenuInitialize;
    //
    if not GetPlaylistManager(FPlaylistManager) then
      FPlaylistManager:=nil;
    //
    FHook:=TAIMPHook.Create;
    FHook.Plugin:=Self;
    CoreUnit.MessageHook(FHook);
  end;
end;

{
procedure TAIMPAddonDemoPlugin.MenuFinalize;
var
  AMenuManager: IAIMPAddonsMenuManager;
begin
  if GetMenuManager(AMenuManager) then
  try
    AMenuManager.MenuRemove(FMenuHandle);
    FMenuHandle := nil;
  finally
    AMenuManager := nil;
  end;
end;

procedure TAIMPAddonDemoPlugin.MenuInitialize;
var
  AMenuInfo: TAIMPMenuItemInfo;
  AMenuManager: IAIMPAddonsMenuManager;
begin
  if GetMenuManager(AMenuManager) then
  try
    ZeroMemory(@AMenuInfo, SizeOf(AMenuInfo));
    AMenuInfo.StructSize := SizeOf(AMenuInfo);
    AMenuInfo.Bitmap := LoadBitmap(HInstance, 'MENUICON');
    AMenuInfo.Caption := PWideChar(_(myPluginName));
    AMenuInfo.Flags := AIMP_MENUITEM_ENABLED;
    AMenuInfo.Proc := @_MenuClick;
    AMenuInfo.UserData := Self;
    FMenuHandle := AMenuManager.MenuCreate(AIMP_MENUID_PLS, @AMenuInfo);
    //
  finally
    AMenuManager := nil;
  end;
end;
}
procedure TAIMPAddonDemoPlugin.PlayerFinalize;
begin
 //FreeAndNil(myLocales);
end;

procedure TAIMPAddonDemoPlugin.PlayerInitialize;
var
  APlayerManager: IAIMPAddonsPlayerManager;
  {
  s: String;
  i: Integer;
  }
begin
  if GetPlayerManager(APlayerManager) then
  try
    APlayerManager.ConfigGetPath(AIMP_CFG_PATH_PROFILE, StaticWideString, MAX_PATH);
    AIMPProfilePath:=WideString(StaticWideString);
    APlayerManager.ConfigGetPath(AIMP_CFG_PATH_PLUGINS, StaticWideString, MAX_PATH);
    PluginsFolder:=WideString(StaticWideString);
    myPluginFolder:=AIMPProfilePath+myPluginName+'\';
    {
    if not DirectoryExists(myPluginFolder) then
      CreateDir(myPluginFolder);
    }
  finally
    APlayerManager := nil;
  end;
end;

function TAIMPAddonDemoPlugin.GetPluginAuthor: PWideChar;
begin
  Result := myPluginAuthor;
end;

function TAIMPAddonDemoPlugin.GetPluginInfo: PWideChar;
begin
  //Result := PWideChar(_(myPluginInfo));
  Result := myPluginInfo;
end;

function TAIMPAddonDemoPlugin.GetPluginName: PWideChar;
begin
  Result := myPluginName;
end;

function TAIMPAddonDemoPlugin.GetPluginFlags: Cardinal;
begin
  Result := AIMP_ADDON_FLAGS_HAS_DIALOG;
end;

function TAIMPAddonDemoPlugin.ShowSettingsDialog(AParentWindow: HWND): HRESULT;
begin
  if not Assigned(SF) then
    SF:=TCTITF_SetupForm.Create(nil);
  if not Assigned(SF.Plugin) then
    SF.Plugin:=Self;
  SF.Show;
  Result := S_OK;
end;

procedure TAIMPAddonDemoPlugin.OnSettingsChanged;
begin
 //menu
 {
 if Assigned(FMenuHandle) then
   MenuFinalize;
 MenuInitialize;
 }
 with mySettings do
  begin
   if Assigned(FFilesList) then
     FreeAndNil(FFilesList);
   FFilesList:=TWideStringList.Create;
   if FileExists(mySettings.Filename) then
     FFilesList.LoadFromFile(mySettings.Filename);
   if UTF then
     FFilesList.SaveFormat:=sfUTF8
   else
     FFilesList.SaveFormat:=sfAnsi;
  end;
end;

procedure TAIMPAddonDemoPlugin.UpdateCurrentTrack(const Text: PWideChar);
 var
   myText: WideString;
begin
 if Text=nil then
   myText:=''
 else
   myText:=Text;
 if mySettings.Bottom then
  begin
   while FFilesList.Count<mySettings.Count do
     FFilesList.Insert(0, '');
   while FFilesList.Count>mySettings.Count do
     FFilesList.Delete(0);
   if WideSameText(FFilesList[mySettings.Count-1], '') then
     FFilesList[mySettings.Count-1]:=myText
   else
    begin
     FFilesList.Add(myText);
     FFilesList.Delete(0);
    end;
  end
 else
  begin
   if FFilesList.Count>0 then
    begin
     if WideSameText(FFilesList[0], '') then
       FFilesList[0]:=myText
     else
       FFilesList.Insert(0, myText);
    end
   else
     FFilesList.Add(myText);
   while FFilesList.Count>mySettings.Count do
     FFilesList.Delete(mySettings.Count);
  end;
 //Save file
 try
  FFilesList.SaveToFile(mySettings.Filename);
 except
 end;
end;

 { TAIMPHook }

procedure TAIMPHook.CoreMessage(AMessage: DWORD; AParam1: Integer; AParam2: Pointer; var AResult: HRESULT); stdcall;
 var
   {
   PLS: HPLS;
   PE: HPLSENTRY;
   i: Integer;
   }
   //myText: Text;
   APlugin: TAIMPAddonDemoPlugin;
   wc: PWideChar;
begin
 APlugin:=TAIMPAddonDemoPlugin(Plugin);
 if not Assigned(APlugin.PlaylistManager) then Exit;
 case AMessage of
  AIMP_MSG_EVENT_STREAM_START, AIMP_MSG_EVENT_STREAM_START_SUBTRACK:
   begin
    (*
    AssignFile(myText, mySettings.Filename);
    {$I-}
    Rewrite(myText);
    {$I+}
    if IOResult=0 then
     begin
      try
       APlugin.PlaylistManager.FormatString(PWideChar(mySettings.Template), Length(mySettings.Template), AIMP_PLAYLIST_FORMAT_MODE_CURRENT, nil, wc);
       Writeln(myText, WideCharToString(wc));
      finally
       CloseFile(myText);
      end;
     end;
    *)
    APlugin.PlaylistManager.FormatString(PWideChar(mySettings.Template), Length(mySettings.Template), AIMP_PLAYLIST_FORMAT_MODE_CURRENT, nil, wc);
    APlugin.UpdateCurrentTrack(wc);
   end;
  AIMP_MSG_EVENT_STREAM_END:
   begin
    {
    try
     DeleteFile(mySettings.Filename);
    except
    end;
    }
    APlugin.UpdateCurrentTrack(nil);
   end;
  AIMP_MSG_EVENT_PLAYER_STATE:
   begin
    APlugin.PlayerState:=AParam1;
   end;
  end;
end;

destructor TAIMPHook.Destroy;
begin
 inherited;
end;

end.
